/***************************************************************************

This macro will allow the caller to do a batch update to the SPM database.

The following options must be set prior to running this macro and should
reference the metadata server where the event information can be picked up
from.  For example:
        metaserver=seshp13
        metaport=8561
        metarepository=Foundation


The following parameters can/must be specified:
        eventName               optional, defaults to the asynchronous event (SAS.Solutions.SpmBOBInterface).  If synchronous
                                          processing is desired, specify the SAS.Solutions.SpmBOBInterfaceSynch event.
        sentBy                  optional, defaults to a descriptive value.
        user                    required, userid of who is performing the update.
        domain                  optional, the domain that the user logs in with.
        pw                      required, the password for the user, it is advisable to use
                                          an encrypted value.  if no password is specified,
                                          use the keyword "nopw".
                                          i.e. - proc pwencode in='pw_to_encrypt'; run;
        entityKey               required if no user/pw is specified,  the entityKey for the users session.
        priority                optional, defaults to a valid value.
        projectname             required, name of the project.
        templatename            required, name of the template that contains the project.


  These variables specify file input and have precedence over data set input.  These
  files must be accessible from the midtier (i.e. use share and \\machinename if this
  SAS job is running on a different machine than the midtier).
        scorecard               optional,  specifies the input file for scorecard data.
        attribute               optional,  specifies the input file for attribute data.
        element                 optional,  specifies the input file for element data.
        cell                    optional,  specifies the input file for cell data.
        cellformat              optional,  specifies the input file for cell format data.
        setup                   optional , specifies the input file for data associated with the template.
                                (element types, metric attributes, templates, attribute definitions).
        project                 optional,  specifies the input file for project data.
        ranges                  optional,  specifies the input file for range data.
        filter                  optional,  specifies the input file for filter data.

  These variables specify data set input.
        scorecardds             optional,  specifies the input data set for scorecard data.
        attributeds             optional,  specifies the input data set for attribute data.
        elementds               optional,  specifies the input data set for element data.
        cellds                  optional,  specifies the input data set for cell data.
        cellformatds            optional,  specifies the input data set for cell format data.
        setupds                 optional,  specifies the input data set for data associated with the template.
                                (element types, metric attributes, templates, attribute definitions).
        projectds               optional,  specifies the input data set for project data.
        rangesds                optional,  specifies the input data set for range data.
        filterds                optional,  specifies the input data set for filter data.
  
        
  If no file or data set input is specified, and if the action is either create or update, then use the appropriately
  named files in the outputdir as input files.  These would be the files named <projectname>_scorecard.csv, 
  <projectname>_attribute.csv, <projectname>_element.csv, <projectname>_cell.csv, <projectname>_setup.csv, 
  <projectname>_ranges.csv, <projectname>_filter.csv.

  This variable specifies the path/directory that SAS should write the converted data
  sets to.  This directory must be accessible from the midtier (i.e. use share and
  \\machinename if this SAS job is running on a different machine than the midtier).  If
  this SAS job is running on the midtier machine then the following parameter is not
  neccessary.
        converteddsdir          optional, specifies the directory the converted, input datasets
                                will be written to.
                        
  If output is desired to be in SAS data set form, specify the following parameter.  This should be an assigned SAS
  libref where the output SAS data sets will be written.  They will be named <projectname>_scorecard, 
  <projectname>_attribute, <projectname>_element, <projectname>_cell, <projectname>_setup, <projectname>_ranges, 
  <projectname>_filter.  This is only enabled when using the synchronous event.
        outputdslib             optional, specifies the SAS data library that the output SAS datasets are stored in.

        outputdir               required, This is the directory BMF (Java) writes output to.
        projectid               optional, the guid of the project to add data under.
        action                  required, defaults to create.  Could also be modify or get.
        resulttype              optional, defaults to 1.  1=guids.  2=relative ids.

  These variables are used for exporting to IMap and are not necessary to be supplied for
  typical BMF functions.
        scorecardid   optional, id of the scorecard to be exported.
        elementtypeid optional, element type id selected at time of export.
        date          optional, date value, if set to current use current date.
        sort          optional, element sorting at time of export.
        filterid      optional, filtering in place at time of exporting.
        cultureid     optional, cultureid at time of exporting.
        namevar       optional, value of the Name column in the output data.
        scorecardvar  optional, value of the Scorecard column in the output data.
        badnumber     optional, value to use in case of a bad number.
****************************************************************************/

%macro spmbmf(eventName=,
                    sentBy=,
                    domain=,
                    user=,
                    pw=,
                    entitykey=x,
                    priority=10,
                    scorecard=,
                    attribute=,
                    element=,
                    cell=,
                    cellformat=,
                    setup=,
                    project=,
                    ranges=,
                    filter=,
                    scorecardds=,
                    attributeds=,
                    elementds=,
                    cellds=,
                    cellformatds=,
                    setupds=,
                    projectds=,
                    rangesds=,
                    filterds=,
                    converteddsdir=,
                    outputdir=c:\,
                    projectid=,
                    projectname=,
                    templatename=,
                    scorecardid=,
                    elementtypeid=,
                    date=,
                    sort=,
                    filterid=,
                    cultureid=,
                    action=create,
                    resulttype=1,
                    namevar=,
                    scorecardvar=,
                    badnumber=,
                    outputdslib=,
                    outputdsdir=);

    %global eventserver appname;
    %local props prop_vals scorecardprop attributeprop elementprop cellprop cellformatprop setupprop projectprop rangesprop filterprop teststr delim outputdirslash;

    %let appname = spmbmf;
    %let outputdir=%bquote(&outputdir);
    
    
    /* convert the parameter values to uppercase */
    %let action = %qupcase(&action);
    
    %if %bquote(&user) = %str() %then %do;
      %put ERROR: Cannot run &appname without specifying a user. ;
          %abort;
    %end;
    %if %bquote(&pw) = %str() %then %do;
      %put ERROR: Cannot run &appname without specifying a password. ;
          %abort;
    %end;
    
    /* shape up the outputdir */
    %if %bquote(&outputdir) = %str() and %bquote(&action) = GET %then %do;
          %put ERROR: Cannot run &appname with a blank output dir. ;
              %abort;
    %end;
    %if %index(&outputdir,\) > 0 %then %do;
    %let delim = \;
    %end;
    %else %do;
      %let delim = /;
    %end;
    %if %index(%substr(&outputdir,%length(&outputdir),1),&delim) = 0 %then %do;
       %let outputdirslash = %trim(&outputdir)&delim;
    %end;
    %else %do;
      %let outputdirslash = &outputdir;
      %let outputdir = %substr(&outputdir,1,%eval(%length(&outputdir)-1));
    %end;

    /* set metauser and metapass options to the SPM user/pw if the metauser option has not already been set */
    %let teststr = %sysfunc(getoption(metauser));
    %if %bquote(&teststr) = %str() %then %do;
      options metauser="&user" metapass="&pw";
    %end;

    /* get a valid event listener to send the event to */
    %getlstnr;
   
   /* if there is no event server then abort */
   %if %bquote(&EVENTSERVER) = %str() %then %do;
     %put ERROR: Cannot run &appname without an event server. ;
         %abort;
   %end;
   
   %if %bquote(&action)=CREATE or %bquote(&action)=UPDATE %then %do;
     /* check for input file/ds values */
     %if %bquote(&scorecard) = %str() and
         %bquote(&attribute) = %str() and
         %bquote(&element) = %str() and
         %bquote(&cell) = %str() and
         %bquote(&cellformat) = %str() and
         %bquote(&setup) = %str() and
         %bquote(&project) = %str() and
         %bquote(&ranges) = %str() and
         %bquote(&filter) = %str() and
         %bquote(&projectds) = %str() and
         %bquote(&scorecardds) = %str() and
         %bquote(&attributeds) = %str() and
         %bquote(&elementds) = %str() and
         %bquote(&cellds) = %str() and
         %bquote(&cellformatds) = %str() and
         %bquote(&setupds) = %str() and
         %bquote(&rangesds) = %str() and
         %bquote(&filterds) = %str() %then %do;

         /* no input files specified, default to files in the output directory if they are present. */
         %if %sysfunc(fileexist(&outputdirslash&projectname._Scorecard.csv)) %then
           %let scorecard=&outputdirslash&projectname._Scorecard.csv;
         %if %sysfunc(fileexist(&outputdirslash&projectname._Element.csv)) %then
           %let element=&outputdirslash&projectname._Element.csv;
         %if %sysfunc(fileexist(&outputdirslash&projectname._Attribute.csv)) %then
           %let attribute=&outputdirslash&projectname._Attribute.csv;
         %if %sysfunc(fileexist(&outputdirslash&projectname._Cell.csv)) %then
           %let cell=&outputdirslash&projectname._Cell.csv;
         %if %sysfunc(fileexist(&outputdirslash&projectname._Setup.csv)) %then
           %let setup=&outputdirslash&projectname._Setup.csv;
         %if %sysfunc(fileexist(&outputdirslash&projectname._Project.csv)) %then
           %let setup=&outputdirslash&projectname._Project.csv;
         %if %sysfunc(fileexist(&outputdirslash&projectname._Ranges.csv)) %then
           %let ranges=&outputdirslash&projectname._Ranges.csv;
         %if %sysfunc(fileexist(&outputdirslash&projectname._Filter.csv)) %then
           %let filter=&outputdirslash&projectname._Filter.csv;
         %if %sysfunc(fileexist(&outputdirslash&projectname._CellFormat.csv)) %then
           %let filter=&outputdirslash&projectname._CellFormat.csv;
     %end;
   %end;
   

   /* convert a supplied SAS ds to a CSV file to be used for input. */
   %if %bquote(&scorecard) = %str() %then %do;
     /* check for a data set */
     %if %bquote(&scorecardds) = %str() %then %do;
           /* no data set either */
           %let scorecardprop=;
           %if %bquote(&debug) = %str(Y) %then %do;
             %put No scorecard input;
           %end;
         %end;
         %else %do;
           /* input is in data set form */
           %let scorecardprop=scorecard;
           %getInputFile(ds=&scorecardds, f=tmpFile, path=&converteddsdir);
           %let scorecard=&tmpFile;
           %if %bquote(&debug) = %str(Y) %then %do;
             %put scorecard input from SAS dataset written to csv: &tmpFile;
           %end;
         %end;
   %end;
   %else %do;
       %let scorecardprop=scorecard;
           %if %bquote(&debug) = %str(Y) %then %do;
             %put scorecard input: &scorecard;
           %end;
   %end;

/* sas ds for attribute info? */
   %if %bquote(&attribute) = %str() %then %do;
     /* check for a data set */
     %if %bquote(&attributeds) = %str() %then %do;
           /* no data set either */
           %let attributeprop=;
           %if %bquote(&debug) = %str(Y) %then %do;
             %put No attribute input;
           %end;
         %end;
         %else %do;
           /* input is in data set form */
       %let attributeprop=attribute;
           %getInputFile(ds=&attributeds, f=tmpFile, path=&converteddsdir);
           %let attribute=&tmpFile;
           %if %bquote(&debug) = %str(Y) %then %do;
             %put attribute input from SAS dataset written to csv: &tmpFile;
           %end;
         %end;
   %end;
   %else %do;
       %let attributeprop=attribute;
           %if %bquote(&debug) = %str(Y) %then %do;
             %put attribute input: &attribute;
           %end;
   %end;

/* sas ds for element info */
   %if %bquote(&element) = %str() %then %do;
     /* check for a data set */
     %if %bquote(&elementds) = %str() %then %do;
           /* no data set either */
           %let elementprop=;
           %if %bquote(&debug) = %str(Y) %then %do;
             %put No element input;
           %end;
         %end;
         %else %do;
           /* input is in data set form */
       %let elementprop=element;
           %getInputFile(ds=&elementds, f=tmpFile, path=&converteddsdir);
           %let element=&tmpFile;
           %if %bquote(&debug) = %str(Y) %then %do;
             %put element input from SAS dataset written to csv: &tmpFile;
           %end;
         %end;
   %end;
   %else %do;
       %let elementprop=element;
           %if %bquote(&debug) = %str(Y) %then %do;
             %put element input: &element;
           %end;
   %end;
   
   /* sas ds for cell format info */
      %if %bquote(&cellformat) = %str() %then %do;
        /* check for a data set */
        %if %bquote(&cellformatds) = %str() %then %do;
              /* no data set either */
              %let cellformatprop=;
              %if %bquote(&debug) = %str(Y) %then %do;
                %put No cell format input;
              %end;
            %end;
            %else %do;
              /* input is in data set form */
          %let cellformatprop=cellformat;
              %getInputFile(ds=&cellformatds, f=tmpFile, path=&converteddsdir);
              %let cellformat=&tmpFile;
              %if %bquote(&debug) = %str(Y) %then %do;
                %put cell format input from SAS dataset written to csv: &tmpFile;
              %end;
            %end;
      %end;
      %else %do;
          %let cellformatprop=cellformat;
              %if %bquote(&debug) = %str(Y) %then %do;
                %put cell format input: &cellformat;
              %end;
   %end;

/* sas ds for cell info */
   %if %bquote(&cell) = %str() %then %do;
     /* check for a data set */
     %if %bquote(&cellds) = %str() %then %do;
           /* no data set either */
           %let cellprop=;
           %if %bquote(&debug) = %str(Y) %then %do;
             %put No cell input;
           %end;
         %end;
         %else %do;
           /* input is in data set form */
       %let cellprop=cell;
           %getInputFile(ds=&cellds, f=tmpFile, path=&converteddsdir);
           %let cell=&tmpFile;
           %if %bquote(&debug) = %str(Y) %then %do;
             %put cell input from SAS dataset written to csv: &tmpFile;
           %end;
         %end;
   %end;
   %else %do;
       %let cellprop=cell;
           %if %bquote(&debug) = %str(Y) %then %do;
             %put cell input: &cell;
           %end;
   %end;

/* sas ds for setup info */
   %if %bquote(&setup) = %str() %then %do;
     /* check for a data set */
     %if %bquote(&setupds) = %str() %then %do;
           /* no data set either */
           %let setupprop=;
           %if %bquote(&debug) = %str(Y) %then %do;
             %put No setup input;
           %end;
         %end;
         %else %do;
           /* input is in data set form */
       %let setupprop=setup;
           %getInputFile(ds=&setupds, f=tmpFile, path=&converteddsdir);
           %let setup=&tmpFile;
           %if %bquote(&debug) = %str(Y) %then %do;
             %put setup input from SAS dataset written to csv: &tmpFile;
           %end;
         %end;
   %end;
   %else %do;
       %let setupprop=setup;
           %if %bquote(&debug) = %str(Y) %then %do;
             %put setup input: &setup;
           %end;
   %end;

/* sas ds for ranges info */
   %if %bquote(&ranges) = %str() %then %do;
     /* check for a data set */
     %if %bquote(&rangesds) = %str() %then %do;
           /* no data set either */
           %let rangesprop=;
           %if %bquote(&debug) = %str(Y) %then %do;
             %put No ranges input;
           %end;
         %end;
         %else %do;
           /* input is in data set form */
       %let rangesprop=range;
           %getInputFile(ds=&rangesds, f=tmpFile, path=&converteddsdir);
           %let ranges=&tmpFile;
           %if %bquote(&debug) = %str(Y) %then %do;
             %put ranges input from SAS dataset written to csv: &tmpFile;
           %end;
         %end;
   %end;
   %else %do;
       %let rangesprop=range;
           %if %bquote(&debug) = %str(Y) %then %do;
             %put ranges input: &ranges;
           %end;
   %end;

/* sas ds for filter info */
   %if %bquote(&filter) = %str() %then %do;
     /* check for a data set */
     %if %bquote(&filterds) = %str() %then %do;
           /* no data set either */
           %let filterprop=;
           %if %bquote(&debug) = %str(Y) %then %do;
             %put No filter input;
           %end;
         %end;
         %else %do;
           /* input is in data set form */
       %let filterprop=filter;
           %getInputFile(ds=&filterds, f=tmpFile, path=&converteddsdir);
           %let filter=&tmpFile;
           %if %bquote(&debug) = %str(Y) %then %do;
             %put filter input from SAS dataset written to csv: &tmpFile;
           %end;
         %end;
   %end;
   %else %do;
       %let filterprop=filter;
           %if %bquote(&debug) = %str(Y) %then %do;
             %put filter input: &filter;
           %end;
   %end;
   
 /* sas ds for project info */
    %if %bquote(&project) = %str() %then %do;
      /* check for a data set */
      %if %bquote(&projectds) = %str() %then %do;
            /* no data set either */
            %let projectprop=;
            %if %bquote(&debug) = %str(Y) %then %do;
              %put No Project input;
            %end;
          %end;
          %else %do;
            /* input is in data set form */
        %let projectprop=project;
            %getInputFile(ds=&projectds, f=tmpFile, path=&converteddsdir);
            %let filter=&tmpFile;
            %if %bquote(&debug) = %str(Y) %then %do;
              %put Project input from SAS dataset written to csv: &tmpFile;
            %end;
          %end;
    %end;
    %else %do;
        %let projectprop=project;
            %if %bquote(&debug) = %str(Y) %then %do;
              %put Project input: &project;
            %end;
   %end;


   /* set the event name and caller name */
   %if %bquote(&sentby) = %str() %then %let sentby=&appname.Caller;
   %if %bquote(&eventName) = %str() %then %let eventName=SAS.Solutions.SpmBOBInterface;

   /* construct the event properties */
   %let props=outputdir bobuser sentby pw entitykey action resulttype;
   %let prop_vals=&outputdir#&user#&sentby#&pw#&entitykey#&action#&resulttype;

   /* add scorecard csv file if present. */
   %if &scorecardprop ne %then %do;
     %let props=&props &scorecardprop;
     %let prop_vals=&prop_vals.#&scorecard;
   %end;

   /* add attribute csv file if present. */
   %if &attributeprop ne %then %do;
     %let props=&props &attributeprop;
     %let prop_vals=&prop_vals.#&attribute;
   %end;

   /* add element csv file if present. */
   %if &elementprop ne %then %do;
     %let props=&props &elementprop;
     %let prop_vals=&prop_vals.#&element;
   %end;
   
    /* add cell format csv file if present. */
      %if &cellformatprop ne %then %do;
        %let props=&props &cellformatprop;
        %let prop_vals=&prop_vals.#&cellformat;
   %end;

   /* add cell csv file if present. */
   %if &cellprop ne %then %do;
     %let props=&props &cellprop;
     %let prop_vals=&prop_vals.#&cell;
   %end;

   /* add setup csv file if present. */
   %if &setupprop ne %then %do;
     %let props=&props &setupprop;
     %let prop_vals=&prop_vals.#&setup;
   %end;

   /* add ranges csv file if present. */
   %if &rangesprop ne %then %do;
     %let props=&props &rangesprop;
     %let prop_vals=&prop_vals.#&ranges;
   %end;

   /* add filter csv file if present. */
   %if &filterprop ne %then %do;
     %let props=&props &filterprop;
     %let prop_vals=&prop_vals.#&filter;
   %end;
   
   /* add project csv file if present. */
      %if &projectprop ne %then %do;
        %let props=&props &projectprop;
        %let prop_vals=&prop_vals.#&project;
   %end;

   /* add domain if present. */
   %if &domain ne %then %do;
     %let props=&props bobdomain;
     %let prop_vals=&prop_vals.#&domain;
   %end;

   /* add projectid if present. */
   %if %nrbquote(&projectid) ne %then %do;
     %let props=&props projectid;
     %let prop_vals=&prop_vals.#&projectid;
   %end;

   /* add scorecardid if present. */
   %if %nrbquote(&scorecardid) ne %then %do;
     %let props=&props scorecardid;
     %let prop_vals=&prop_vals.#&scorecardid;
   %end;

   /* add elementtypeid if present. */
   %if %nrbquote(&elementtypeid) ne %then %do;
     %let props=&props elementtypeid;
     %let prop_vals=&prop_vals.#&elementtypeid;
   %end;

   /* add date if present. */
   %if &date ne current and &date ne %then %do;
     %let props=&props date;
     %let prop_vals=&prop_vals.#&date;
   %end;

   /* add sort if present. */
   %if &sort ne %then %do;
     %let props=&props sort;
     %let prop_vals=&prop_vals.#&sort;
   %end;

   /* add filterid if present. */
   %if %nrbquote(&filterid) ne %then %do;
     %let props=&props filterid;
     %let prop_vals=&prop_vals.#&filterid;
   %end;

   /* add cultureid if present. */
   %if %nrbquote(&cultureid) ne %then %do;
     %let props=&props cultureid;
     %let prop_vals=&prop_vals.#&cultureid;
   %end;

   /* add projectname if present. */
   %if %nrbquote(&projectname) ne %then %do;
     %let props=&props projectname;
     %let prop_vals=&prop_vals.#%nrbquote(&projectname);
   %end;

   /* add templatename if present. */
   %if %nrbquote(&templatename) ne %then %do;
     %let props=&props templatename;
     %let prop_vals=&prop_vals.#%nrbquote(&templatename);
   %end;

   /* add namevar if present. */
   %if %nrbquote(&namevar) ne %then %do;
     %let props=&props namevar;
     %let prop_vals=&prop_vals.#&namevar;
   %end;

   /* add scorecardvar if present. */
   %if %nrbquote(&scorecardvar) ne %then %do;
     %let props=&props scorecardvar;
     %let prop_vals=&prop_vals.#&scorecardvar;
   %end;

   /* add badnumber if present. */
   %if %nrbquote(&badnumber) ne %then %do;
     %let props=&props badnumber;
     %let prop_vals=&prop_vals.#%nrbquote(&badnumber);
   %end;

   %if %bquote(&debug) = %str(Y) %then %do;
     %put props:     &props;
     %put prop_vals: &prop_vals;
   %end;
   
   %local result_url;
   %if &eventName=SAS.Solutions.SpmBOBInterfaceSynch %then %do;
     filename foo temp;
     %let result_url = %bquote(%sysfunc(pathname(foo)));
   %end;

   %if &eventName=SAS.Solutions.SpmBOBInterfaceSynch %then %do;
     filename foo temp;
     %let result_url = %bquote(%sysfunc(pathname(foo)));
     %if %bquote(&debug) = %str(Y) %then %do;
      %put Going to send a synchronous event now.;
     %end;

     %sendevnt(evntname=&eventName,
               priority=&priority,
               eventserver=&EVENTSERVER,
               sentby=&sentby,
               response=RESULT,
               result_url=&result_url,
               props=%bquote(&props),
               vals=%bquote(&prop_vals));
   %end;
   %else %do;
     %if %bquote(&debug) = %str(Y) %then %do;
      %put Going to send an asynchronous event now.;
     %end;
     %sendevnt(evntname=&eventName,
               priority=&priority,
               eventserver=&EVENTSERVER,
               sentby=&sentby,
               props=%bquote(&props),
               vals=%bquote(&prop_vals));
   %end;
   
   /* check to see if csv output needs to be converted to datasets */
   %if %bquote(&eventname) = %str(SAS.Solutions.SpmBOBInterfaceSynch) and %bquote(&outputdslib) ne %str() %then %do;
     %if %bquote(&debug) = %str(Y) %then %do;
      %put SAS dataset output was requested, converting csv files to datasets.;
     %end;

     /* it does, so check for each csv output file and convert it if it is present. */
     %if %sysfunc(fileexist(&outputdirslash&projectname._Scorecard.csv)) %then %do;
       %if %bquote(&debug) = %str(Y) %then %do;
          %put Converting scorecard csv file to dataset.;
       %end;
       proc import DATAFILE="&outputdirslash&projectname._Scorecard.csv"
                   OUT=&outputdslib..&projectname._Scorecard
                   DBMS=CSV
                   REPLACE;
       run;
     %end;
     %else %do;
       %if %bquote(&debug) = %str(Y) %then %do;
         %put no scorecard csv file to convert.;
        %end;
     %end;
     
     %if %sysfunc(fileexist(&outputdirslash&projectname._Element.csv)) %then %do;
       %if %bquote(&debug) = %str(Y) %then %do;
          %put Converting element csv file to dataset.;
       %end;
       proc import DATAFILE="&outputdirslash&projectname._Element.csv"
                   OUT=&outputdslib..&projectname._Element
                   DBMS=CSV
                   REPLACE;
       run;
     %end;
     %else %do;
       %if %bquote(&debug) = %str(Y) %then %do;
         %put no element csv file to convert.;
       %end;
    %end;
    
    %if %sysfunc(fileexist(&outputdirslash&projectname._Attribute.csv)) %then %do;
       %if %bquote(&debug) = %str(Y) %then %do;
          %put Converting attribute csv file to dataset.;
       %end;
       proc import DATAFILE="&outputdirslash&projectname._Attribute.csv"
                   OUT=&outputdslib..&projectname._Attribute
                   DBMS=CSV
                   REPLACE;
       run;
     %end;
     %else %do;
       %if %bquote(&debug) = %str(Y) %then %do;
         %put no attribute csv file to convert.;
       %end;
     %end;
     
     %if %sysfunc(fileexist(&outputdirslash&projectname._Cell.csv)) %then %do;
       %if %bquote(&debug) = %str(Y) %then %do;
          %put Converting cell csv file to dataset.;
       %end;
       proc import DATAFILE="&outputdirslash&projectname._Cell.csv"
                   OUT=&outputdslib..&projectname._Cell
                   DBMS=CSV
                   REPLACE;
       run;
     %end;
     %else %do;
       %if %bquote(&debug) = %str(Y) %then %do;
         %put no cell csv file to convert.;
       %end;
     %end;
     
     %if %sysfunc(fileexist(&outputdirslash&projectname._Setup.csv)) %then %do;
       %if %bquote(&debug) = %str(Y) %then %do;
          %put Converting setup csv file to dataset.;
       %end;
       proc import DATAFILE="&outputdirslash&projectname._Setup.csv"
                   OUT=&outputdslib..&projectname._Setup
                   DBMS=CSV
                   REPLACE;
       run;
     %end;
     %else %do;
       %if %bquote(&debug) = %str(Y) %then %do;
         %put no setup csv file to convert.;
       %end;
     %end;
     
     %if %sysfunc(fileexist(&outputdirslash&projectname._Ranges.csv)) %then %do;
       %if %bquote(&debug) = %str(Y) %then %do;
          %put Converting ranges csv file to dataset.;
       %end;
       proc import DATAFILE="&outputdirslash&projectname._Ranges.csv"
                   OUT=&outputdslib..&projectname._Ranges
                   DBMS=CSV
                   REPLACE;
       run;
     %end;
     %else %do;
       %if %bquote(&debug) = %str(Y) %then %do;
         %put no ranges csv file to convert.;
       %end;
     %end;
     
     %if %sysfunc(fileexist(&outputdirslash&projectname._Project.csv)) %then %do;
            %if %bquote(&debug) = %str(Y) %then %do;
               %put Converting ranges csv file to dataset.;
            %end;
            proc import DATAFILE="&outputdirslash&projectname._Project.csv"
                        OUT=&outputdslib..&projectname._Project
                        DBMS=CSV
                        REPLACE;
            run;
          %end;
          %else %do;
            %if %bquote(&debug) = %str(Y) %then %do;
              %put no project csv file to convert.;
            %end;
     %end;
     
     %if %sysfunc(fileexist(&outputdirslash&projectname._Filter.csv)) %then %do;
       %if %bquote(&debug) = %str(Y) %then %do;
          %put Converting filter csv file to dataset.;
       %end;
       proc import DATAFILE="&outputdirslash&projectname._Filter.csv"
                   OUT=&outputdslib..&projectname._Filter
                   DBMS=CSV
                   REPLACE;
       run;
     %end;
     %else %do;
       %if %bquote(&debug) = %str(Y) %then %do;
         %put no filter csv file to convert.;
       %end;
     %end;
   %end;

%mend;



%macro getInputFile(ds=, f=, path=);
%local &f;

%if %bquote(&debug) = %str(Y) %then %do;
  %put in getInputFile(ds=&ds, f=&f, path=&path);
%end;

%local i word source notes optname fid;

%if %bquote(&path) = %str() %then %do;
    %let &f=!SASROOT/soltnsdata/sasmisc/&ds..csv;
%end;
%else %do;
    %let test = %substr(&path, %length(&path), 1);
    %if %bquote(&test) = %str(/) or %bquote(&test) = %str(\) %then %do;
      %let &f=&path.&ds..csv;
    %end;
    %else %do;
      %let &f=&path/&ds..csv;
    %end;
%end;

filename tmpfile "&&&f";

/* get full, resolved path name */
%let fid=%sysfunc(fopen(tmpfile));
%if &fid gt 0 %then %do;
    %let optname=%sysfunc(foptname(&fid, 1));
    %let &f=%sysfunc(finfo(&fid,&optname));
    %let fid=%sysfunc(fclose(&fid));
%end;
%if %bquote(&debug) = %str(Y) %then %do;
%put file to write out to is &&&f;
%end;

%getvars(dsname=&ds);
options notes missing=' ';
data _null_;
set &ds;
file tmpfile LINESIZE=32767;
if _N_ = 1 then do;
/* put out a dummy record at the top since java will discard it. */
put "please ignore this row, java will consume it.";
end;

%let i = 1;
%do %until (&word=%str());
%let word=%scan(&vars, &i, %str( ));
    %if &word ne %str() %then %do;
  if index(&word, ',') > 0 then
    put '"' &word +(-1)'",' @;
      else if &word = '' then
        put ',' @;
      else
        put &word  +(-1)',' @;
    %end;
    %let i = %eval(&i + 1);
%end;
put +(-1)' ';
run;
options missing='.';
filename tmpfile clear;
%mend;

    %macro getvars(dsname=,pw=);
      %local i n;
      %global vars;
      %let n=0;
      %let vars=;
      %if &pw ne %then %let pw=read=&pw;
      options nodsnferr nonotes nosource;
      proc contents varnum data=&dsname(&pw) noprint out=_varlst_(keep=name varnum);
      proc sort data=_varlst_; by varnum;
      run;
      data _null_;
        set _varlst_ end=end;
        call symput('n'!!left(put(_n_,3.)),name);
        if end then call symput('n',left(put(_n_,3.)));
      run;
      proc datasets lib=work nolist;delete _varlst_;quit;
      options &notes &source;
      %do i=1 %to &n;
        %let vars=%trim(&vars) &&n&i;
      %end;
      %let vars=%str( &vars );
      options notes source;
    %mend;

/*

Here's an example invocation of this macro

options metaserver=<server-name> metaport=8561 metarepository=Foundation;
%let debug=Y;
%spmbmf(eventName=,
              action=create,
              sentBy=,
              user=<user_name>,
              pw=<password>,
	      templatename=<template_name>,
	      projectName=<project_name,
              setup=<setup_file_path_and_filename>,
	      project=<project_file_path_and_filename>,
	      ranges=<range_file_path_and_filename>,
	      outputdir=<output_file_path>
              );
 */

/* run this to get your encrypted password */
/* proc pwencode in='pw_to_encrypt'; run; */

/* sample data */
/*
data attribute;
input @1 var1 $1. @3 var2 $12. @16 var3 $9. @26 var4 $21.;
cards;
1 text         MyText    MyLabel
1 email        MyEmail   John.Doe@yahoo.com
1 date         MyDate    1/24/2006
1 link         MyLink    www.yahoo.com
1 Element Type MyElement 3
;
run;
*/


